//=============================================================================
//                                                              SynergyEA.mq4
//                                              Copyright  2007, Derk Wehler
//
//=============================================================================
#property copyright "Copyright  2007, Derk Wehler"
#property link      "http://www.ArrogantFxBastards.com"

#include <LibOrderReliable.mqh>
#include <LibDerksUtils.mqh>


// Input parameters
extern string	S1 					= "== MONEY MANAGEMENT SETTINGS ==";
extern bool 	UseMoneyManagement 	= false;
extern bool		UseDualWinMM		= false;
extern double	TradeSizePercent 	= 3.0;	// Change to whatever percent of equity you wish to risk.
extern double	ScaleInPercent	 	= 20.0;
extern double	MaxScaleIns			= 3;
extern double	Lots 				= 0.1;
extern double	MinLots 			= 0.01;
extern double	MaxLots 			= 100.0;
extern string	S1z 				= " ";
extern string	S2 					= "==== INDICATOR SETTINGS ====";
extern string	S2a 				= "--------- INDICATOR ---------";
extern string	S2b 				= "------ Entry Settings ------";
extern int		UseEntry68_32		= 0;
extern int		UseSmallerExit		= 0;
extern double	DefineSmaller		= 0.08;
extern int		ReqRedYellowCombo	= 0;
extern int		UseVolExpanding		= 1;
extern int		UseChaikin			= 0;
extern int		UseH4Trend			= 0;
extern string	S2c 				= "------ Exit Settings -------";
extern int		UseSmallerOrOpp		= 0;
extern int		UseClosedInside		= 0;
extern int		UseTDI_ExitDir		= 0;
extern int		UseOppPAC_Trail		= 1;
extern string	S2z 				= " ";
extern string	S3 					= "==== EA SETTINGS ====";
extern string	S3a 				= "------ SL, TP & Trail ------";
extern int		StopLoss	 		= 0;
extern int		TakeProfit	 		= 0;
extern int	 	TrailingType 		= 0;
extern int	 	TrailingValue 		= 0;
extern int	 	FirstMove 			= 0;
extern int	 	FirstSL		 		= 0;
extern int	 	SecondMove 			= 0;
extern int	 	SecondSL	 		= 0;
extern int	 	ThirdMove 			= 0;
extern string	S3b 				= "------ Other Settings ------";
extern bool		UseAlert			= false;
extern bool		UseEmail			= false;
extern string	S3z 				= " ";
extern string	S4c 				= "------ GENERAL EA ------";
extern int		Slippage			= 1;
extern int		MagicSeed			= 23000;
extern string	ExpertName			= "SynergyEA";
extern int		DebugLevel 			= 5;


int 	MagicNumber;
double 	LotsMM;
double	Spread;
int		ScaleIns = 0;
bool	OkToAddPositions = true;

// Indicator values
double	HA_Open_0, HA_Close_0;
double 	HA_Open_1, HA_Close_1;
double 	HA_Open_2, HA_Close_2;
double	HA_DM0_2, HA_DM1_2;
double	BlueH_0, Yellow_0, BlueL_0, Green_0, Red_0;
double	BlueH_1, Yellow_1, BlueL_1, Green_1, Red_1, Red_2;
double	MA_Hi_0, MA_Lo_0;
double	MA_Hi_1, MA_Lo_1;
double	MA_Hi_2, MA_Lo_2;
double	Chaikin;

int		TDI_Dir;
int		MA_Trend_Dir;
int		H4_Trend;
int		HA_Outside_MA_Dir;
bool	HA_Closed_Inside_MA;
int		CandleSmallerOrOpp;
string	ExitReason;

//=============================================================================
//	expert initialization function
//=============================================================================
int init()
{
	// Set up magic numbers:
    MagicNumber = MagicSeed + SymbolConst2Val(Symbol())*100 + TimeFrameConst2Val(Period());
	DU_SetErrorLevel(DebugLevel);

	return (0);
}
  

//=============================================================================
//	expert deinitialization function
//=============================================================================
int deinit()
{
	return (0);
}


//=============================================================================
//
// PURPOSE:
//     Calculate the number of lots to trade.
//
// PARAMETERS:
//	   magic: The magic number used to trade (used by DualWin system only)
//
// RETURN VALUE:
//     The number of lots to trade
//
//=============================================================================
double CalcLots(int magic)
{
	double lots = Lots;

	if (UseMoneyManagement)
	{
		lots = CalcLotsGivenRisk(TradeSizePercent, MinLots, MaxLots, StopLoss);
//		DebugPrint("Using MoneyManagement, lots becomes: " + lots, 1);
	}
	if (UseDualWinMM)
	{
		if (TakeProfit == 0)
			Alert("Attempt to use DualWin MM system with zero TP");
		lots = GetDualWinLots(lots, magic, TakeProfit);
//		DebugPrint("Using Binary, lots becomes: " + calcUnits, 1);
	}
	
	return(lots);
}


//=============================================================================
//
// PURPOSE:
//     Enter a trade with the given parameters
//
// PARAMETERS:
//	     op: The direction to enter trade
//     lots: The percentage to use if MM is enabled
//
// RETURN VALUE:
//     The ticket number of the trade entered
//
//=============================================================================
int DoEntry(int op, double lots)
{
	int 	mult, ticket;
	color	clr;
	double 	price, sl, tp;

	sl = 0;
	tp = 0;
	mult = 1;
	price = Ask;
	clr = Green;
	
	if (op == OP_SELL)
	{
		mult = -1;
		price = Bid;
		clr = Red;
	}
	
	if (StopLoss > 0)
		sl = price - mult * StopLoss*Point;
	if (TakeProfit > 0)
		tp = price + mult * TakeProfit*Point;
		
	ticket = OrderSendReliable(Symbol(), op, lots, price, Slippage, sl, tp, 
								ExpertName, MagicNumber, 0, clr);
}


//=============================================================================
//
// PURPOSE:
//     Exit all trades in a given direction
//
// PARAMETERS:
//	     op: The direction to enter trade
//     lots: The percentage to use if MM is enabled
//
// RETURN VALUE:
//     true: A trdae was exited
//    false: No trade exited
//
//=============================================================================
bool DoExit(int op)
{
	int		cnt;
	bool	somethingClosed = false;
	color	clr;
	double 	price, sl, tp;

	for (cnt=OrdersTotal()-1; cnt >= 0; cnt--)
	{
		OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
		if (OrderSymbol() != Symbol() || 
			OrderMagicNumber() != MagicNumber || 
			OrderType() != op) 
			continue;
		
		price = Bid;
		clr = Green;
	
		if (op == OP_SELL)
		{
			price = Ask;
			clr = Red;
		}
	
		if (OrderCloseReliable(OrderTicket(), OrderLots(), price, Slippage, clr))
			somethingClosed = true;
	}
	return(somethingClosed);
}


//=============================================================================
//	Expert start function
//=============================================================================
int start()
{
	static bool InBuy  = false;
	static bool InSell = false;

	int 	ticket, op1, op2, op3, openPos;
	double	lotsMM;
	
	// Handle triling stop on every pip
	if (TrailingType > 0 && NumOpenPositions(MagicNumber, OP_BOTH) > 0)
		AdjTrailOnAllOrders(TrailingType, TrailingValue, MagicNumber, OP_BOTH, 
							FirstMove, FirstSL, SecondMove, SecondSL, ThirdMove);

	// Check for exits once per candle
	Spread = Ask - Bid;
	if (IsNewCandle())
	{
		OkToAddPositions = true;
		
		// Check for exit signals --------------------------------------------
		HA_Closed_Inside_MA = GetHA_ClosedInside();
		TDI_Dir = GetTDI_ExitDir();
		CandleSmallerOrOpp = GetCandleSmallerOrOpposite();
		
		// If all in Short position, signal to EXIT LONG at this candle's open
		if (InBuy)
		{
			if ((CandleSmallerOrOpp == OP_SELL && UseSmallerOrOpp) || 
				(HA_Closed_Inside_MA && UseClosedInside) || 
				(TDI_Dir == OP_SELL && UseTDI_ExitDir) || 
				(PriceReachedOppositePAC() == OP_SELL && UseOppPAC_Trail))
			{
				Print("                                      Got EXIT BUY signal: " + ExitReason);
				DoAlert("Synergy: Exit Buy Signal: " + ExitReason);
				if (DoExit(OP_BUY))
					InBuy = false;
			}
		}
					
		// If all in Long position, signal to EXIT SHORT position at this candle's open
		if (InSell)
		{
			if ((CandleSmallerOrOpp == OP_BUY && UseSmallerOrOpp) || 
				(HA_Closed_Inside_MA && UseClosedInside) || 
				(TDI_Dir == OP_BUY && UseTDI_ExitDir) || 
				(PriceReachedOppositePAC() == OP_BUY && UseOppPAC_Trail))
			{
				Print("                                      Got EXIT SELL signal: " + ExitReason);
				DoAlert("Synergy: Exit Sell Signal: " + ExitReason);
				if (DoExit(OP_SELL))
					InSell = false;
			}
		}
	}
		

	AcquireIndicatorValues(0);
	
	// Check for entry signals ---------------------------------------------
	MA_Trend_Dir = GetMA_Trend();
	HA_Outside_MA_Dir = GetHA_CloseDir();
	TDI_Dir = GetTDI_EntryDir();
	H4_Trend = GetH4Trend();
	
	// If all in Long position, signal to BUY at this candle's open
	if (!InBuy && MA_Trend_Dir == OP_BUY && HA_Outside_MA_Dir == OP_BUY && 
		TDI_Dir == OP_BUY && (!UseVolExpanding || VolBandsExpanding()) && 
		(!UseChaikin || Chaikin > 0) && (!UseH4Trend || H4_Trend == OP_BUY))
	{
		// Open new position
		lotsMM = CalcLots(MagicNumber);
		DoEntry(OP_BUY, lotsMM);

		InBuy = true;
		DoAlert("SynergyEA: New Buy Signal");
	}
	
	else if (InBuy && AddToBuyPosition())
	{
		// Add to position if we already have one open in this direction
		openPos = NumOpenPositions(MagicNumber, OP_BUY);
		if (openPos > 0 && openPos <= MaxScaleIns && OkToAddPositions)
		{
			lotsMM = CalcLots(MagicNumber) * ScaleInPercent / 100;
			lotsMM = NormalizeDouble(lotsMM, 2);
			DoEntry(OP_BUY, lotsMM);
			OkToAddPositions = false;
		}

		DoAlert("SynergyEA: Add to Buy Signal");
	}
	
	// If all in Short position, signal to SELL at this candle's open
	if (!InSell && MA_Trend_Dir == OP_SELL && HA_Outside_MA_Dir == OP_SELL && 
		TDI_Dir == OP_SELL && (!UseVolExpanding || VolBandsExpanding()) && 
		(!UseChaikin || Chaikin > 0) && (!UseH4Trend || H4_Trend == OP_SELL))
	{
		// Open new position
		lotsMM = CalcLots(MagicNumber);
		DoEntry(OP_SELL, lotsMM);

		InSell = true;
		DoAlert("SynergyEA: New Sell Signal");
	}

	else if (InSell && AddToSellPosition())
	{
		// Add to position if we already have one open in this direction
		openPos = NumOpenPositions(MagicNumber, OP_SELL);
		if (openPos > 0 && openPos <= MaxScaleIns && OkToAddPositions)
		{
			lotsMM = CalcLots(MagicNumber) * ScaleInPercent / 100;
			lotsMM = NormalizeDouble(lotsMM, 2);
			DoEntry(OP_SELL, lotsMM);
			OkToAddPositions = false;
		}

		DoAlert("SynergyEA: Add to Sell Signal");
	}
	

	return (0);
}


void DoAlert(string str)
{
	if (UseAlert)
		Alert(str);

	if (UseEmail)
		SendMail("SynergyInd", str);
}


void AcquireIndicatorValues(int i) 
{
	// Get Haiken Ashi values =========================================================================================
	// Current Candle...
	HA_Open_0 	= iCustom(NULL, 0, "HeikenAshi_DM",   2, i); // Open
	HA_Close_0 	= iCustom(NULL, 0, "HeikenAshi_DM",   3, i); // Close
	
	// Last Closed Candle...
	HA_Open_1 	= iCustom(NULL, 0, "HeikenAshi_DM",   2, i+1); // Open
	HA_Close_1 	= iCustom(NULL, 0, "HeikenAshi_DM",   3, i+1); // Close
	
	// Prev Prev Candle...
	HA_Open_2 	= iCustom(NULL, 0, "HeikenAshi_DM",   2, i+2); // Open
	HA_Close_2 	= iCustom(NULL, 0, "HeikenAshi_DM",   3, i+2); // Close
	

	// Get Traders Dynamic Index values ===============================================================================
	// Current Candle...
	BlueH_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   1, i);		// Blue   (H. Vol Band)
	Yellow_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   2, i);		// Yellow (Market Base)
	BlueL_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   3, i);		// Blue   (L. Vol Band)
	Green_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   4, i);		// Green  (RSI Price)
	Red_0 		= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   5, i);		// Red    (Trade Signal)
	
	// Last Closed Candle...
	BlueH_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   1, i+1);	// Blue   (H. Vol Band)
	Yellow_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   2, i+1);	// Yellow (Market Base)
	BlueL_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   3, i+1);	// Blue   (L. Vol Band)
	Green_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   4, i+1);	// Green  (RSI Price)
	Red_1 		= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   5, i+1);	// Red    (Trade Signal)
	
	// Prev Prev Candle...
	Red_2 		= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   5, i+2);	// Red    (Prev Prev Candle)

	
	// Get Smoothed Moving Average values =============================================================================
	MA_Hi_0 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_HIGH, i);
	MA_Lo_0 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_LOW,  i);
	MA_Hi_1 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_HIGH, i+1);
	MA_Lo_1 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_LOW,  i+1);
	MA_Hi_2 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_HIGH, i+2);
	MA_Lo_2 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_LOW,  i+2);


	// Get Chaikin's Volatility =======================================================================================
	if (UseChaikin == 1)
	{
		//Chaikin	= iCustom(NULL, 0, "Chaikin\'s Volatility", 13, 13, 26,   0, i);	// Red line
		Chaikin	= iCustom(NULL, 0, "Chaikin\'s Volatility_Kalenzo", 13, 13, 2000,   0, i);	// Red line
	}
}


int GetMA_Trend()
{
	int sum = 0;
	
	// We are looking for the general trend here, so check 
	// whether lines are headed up or down, comparing this 
	// candle with the prev and the prev prev
	if (MA_Hi_0 > MA_Hi_1)	sum++;
	if (MA_Lo_0 > MA_Lo_1)	sum++;
	if (MA_Hi_0 > MA_Hi_2)	sum++;
	if (MA_Lo_0 > MA_Lo_2)	sum++;
	if (MA_Hi_1 > MA_Hi_2)	sum++;
	if (MA_Lo_1 > MA_Lo_2)	sum++;
		
	if (MA_Hi_0 < MA_Hi_1)	sum--;
	if (MA_Lo_0 < MA_Lo_1)	sum--;
	if (MA_Hi_0 < MA_Hi_2)	sum--;
	if (MA_Lo_0 < MA_Lo_2)	sum--;
	if (MA_Hi_1 < MA_Hi_2)	sum--;
	if (MA_Lo_1 < MA_Lo_2)	sum--;

	// If we have 3 (or 4??) out of 6, consider it to have a direction
	if (sum >= 3)
		return(OP_BUY);
	else if (sum <= -3)
		return(OP_SELL);
	else
		return(-1);
}


int GetH4Trend()
{
	int		dir0 = -1;
	int		dir1 = -1;
	int		dir2 = -1;
	
	if (HA_Open_0 < HA_Close_0) 	dir0 = OP_BUY;
	else 
	if (HA_Open_0 > HA_Close_0)		dir0 = OP_SELL;
		
	if (HA_Open_1 < HA_Close_1) 	dir1 = OP_BUY;
	else 
	if (HA_Open_1 > HA_Close_1)		dir1 = OP_SELL;
		
	if (HA_Open_2 < HA_Close_2) 	dir2 = OP_BUY;
	else 
	if (HA_Open_2 > HA_Close_2)		dir2 = OP_SELL;
	
	if (dir0 == OP_BUY && dir1 == OP_BUY && dir2 == OP_BUY)		return(OP_BUY);
	else 
	if (dir0 == OP_SELL && dir1 == OP_SELL && dir2 == OP_SELL)	return(OP_SELL);

	return(-1);
}


int GetHA_CloseDir()
{
	if (HA_Close_0 > HA_Open_0 && HA_Close_0 > MA_Hi_0)
	{
		return(OP_BUY);
	}
	else 
	if (HA_Close_0 < HA_Open_0 && HA_Close_0 < MA_Lo_0)
	{
		return(OP_SELL);
	}
	return(-1);
}


bool VolBandsExpanding()
{
	if (BlueH_0 - BlueL_0 > BlueH_1 - BlueL_1)
		return(true);
	return(false);
}


int GetTDI_EntryDir()
{
	if (Green_0 > 50.0 && 
		(Green_0 < 68.0  || !UseEntry68_32) && 
		(Green_0 > Yellow_0 && Green_0 > Red_0) && 
		(Red_0 > Red_1) && //Red_1 > Red_2) && 
		(Red_0 > Yellow_0 || !ReqRedYellowCombo))
		return(OP_BUY);
	else 
	if (Green_0 < 50.0 && 
		(Green_0 > 32.0 || !UseEntry68_32) && 
		(Green_0 < Yellow_0 && Green_0 < Red_0) && 
		(Red_0 < Red_1) && //Red_1 < Red_2) && 
		(Red_0 < Yellow_0 || !ReqRedYellowCombo))
		return(OP_SELL);
		
	return(-1);
}


int GetTDI_ExitDir()
{
	// Check for exiting Buy position
	if (Green_1 > 68.0 && Green_1 < BlueH_1 && Green_1 < Red_1)
	{
		ExitReason = "TDI";
		return(OP_SELL);
	}
	else 
	// Check for exiting sell position
	if (Green_1 < 32.0 && Green_1 > BlueL_1 && Green_1 > Red_1)
	{
		ExitReason = "TDI";
		return(OP_BUY);
	}
	return(-1);
}


bool GetHA_ClosedInside()
{
	// HA_Close_1 is close price
	if (HA_Close_1 > MA_Lo_1 && HA_Close_1 < MA_Hi_1)
	{
		ExitReason = "CI";
		return(true);
	}
	else 
		return(false);
}


int GetCandleSmallerOrOpposite()
{
	if (HA_Close_2 > HA_Open_2)		// 2 candles back was long
	{
		// If prev candle was short, return OP_SELL signal to exit buy
		if (HA_Close_1 < HA_Open_1)
		{
			ExitReason = "OC";
			return(OP_SELL);
		}
			
		// If prev candle was "much smaller" than prev prev, then exit buy
		if (UseSmallerExit == 1)
		{
			if (HA_Close_1 - HA_Open_1 < (HA_Close_2 - HA_Open_2) * DefineSmaller)
			{
				ExitReason = "SC";
				return(OP_SELL);
			}
		}
	}
	else if (HA_Close_2 < HA_Open_2)	// 2 candles back was short
	{
		// If prev candle was long, return OP_BUY signal to exit sell
		if (HA_Close_1 > HA_Open_1)
		{
			ExitReason = "OC";
			return(OP_BUY);
		}
					
		// If prev candle was "much smaller" than prev prev, then exit buy
		if (UseSmallerExit == 1)
		{
			if (HA_Open_1 - HA_Close_1 < (HA_Open_2 - HA_Close_2) * DefineSmaller)
			{
				ExitReason = "SC";
				return(OP_BUY);
			}
		}
	}
	return(-1);
}


int PriceReachedOppositePAC()
{
	if (Close[1] >= MA_Hi_1 + Spread)
	{
		ExitReason = "PAC";
		return(OP_BUY);		// return OP_BUY signal to exit sell
	}
	
	if (Close[1] <= MA_Lo_1 - Spread)
	{
		ExitReason = "PAC";
		return(OP_SELL);	// return OP_SELL signal to exit buy
	}
	return(-1);
}


bool AddToBuyPosition()
{
	if (Green_1 > 50.0 && Green_1 < 68.0 && Green_1 > BlueH_1)
		return(true);
	return(false);
}


int AddToSellPosition()
{
	if (Green_1 < 50.0 && Green_1 > 32.0 && Green_1 < BlueL_1)
		return(true);
	return(false);
}


